/*

Simplified BSD License
taken from
<http://www.opensource.org/licenses/bsd-license.php> 

Copyright (c) 2008, Saleem N. Bhatti <http://www.cs.st-andrews.ac.uk/~saleem/>
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  - Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  - Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

  - Neither the name of 'Saleem N. Bhatti' or 'Saleem Bhatti', nor the
    names of any other contributors may be used to endorse or promote
    products derived from this software without specific prior written
    permission.

DISCLAIMER
----------
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

27 November 2008

*/

/******************************************************************************
Some logging stuff

saleem <http://www.cs.st-andrews.ac.uk/~saleem/>
November 2008
November 2004
November 2001
March 1993
******************************************************************************/

#ifndef __d_Log_h__
#define __d_Log_h__

#include <stdio.h>

class d_Log {

  char *program_;      /* program name */
  char *name_;         /* name of file */
  FILE *log_;          /* non-NULL after call to d_log_open() */

                         /* use the following two for what, flags and syslog */
#define D_LOG_all        ((unsigned char) 0xff)
#define D_LOG_none       ((unsigned char) 0x00)

  unsigned char what_; /* what to log - use as you like */
#define D_LOG_debug      ((unsigned char) 0x01)
#define D_LOG_advise     ((unsigned char) 0x02)
#define D_LOG_error      ((unsigned char) 0x04)
#define D_LOG_trace      ((unsigned char) 0x08)
#define D_LOG_exceptions ((unsigned char) 0x10)
#define D_LOG_fatal      ((unsigned char) 0x20)

  unsigned char syslog_; /* takes same values as what */

  unsigned char flags_;
#define D_LOG_log        ((unsigned char) 0x01) /* log to file 'name' */
#define D_LOG_stderr     ((unsigned char) 0x02) /* log to stderr */
#define D_LOG_close      ((unsigned char) 0x04) /* close between writes */
#define D_LOG_alert      ((unsigned char) 0x08) /* d_Log errors to stderr */
#define D_LOG_time       ((unsigned char) 0x10) /* time */
#define D_LOG_date       ((unsigned char) 0x20) /* date */
#define D_LOG_user       ((unsigned char) 0x40) /* user id */

  unsigned long size_;   /* max log size (bytes) - 0 means no size limit */

  d_Log(const char *program, const char *name,
	const unsigned char what, const unsigned char syslog,
	const unsigned char flags, const unsigned long size);

public:

  static
  d_Log *d_make(const char *program, const char *name,
		const unsigned char what = D_LOG_all,
		const unsigned char syslog = D_LOG_none,
		const unsigned char flags = D_LOG_all,
		const unsigned int long = 0);

  ~d_Log();

  /* Do need not use d_open() and d_close() if using the D_LOG_close flag */
  void d_open();
  void d_close();
  void d_log(const unsigned char what, const char *format, ...);

  /* The methods below are 'just in case ...' */
  char *program();
  char *name();

  unsigned char d_what();
  unsigned char d_syslog();
  unsigned char d_flags();
  unsigned long d_size();

  unsigned char d_what(const unsigned char what);
  unsigned char d_syslog(const unsigned char syslog);
  unsigned char d_flags(const unsigned char flags);
  unsigned long d_size(const unsigned long size);
};

inline char *
d_Log::program() { return program_; }

inline char *
d_Log::name() { return name_; }

inline unsigned char
d_Log::d_what() { return what_; }

inline unsigned char
d_Log::d_flags() { return flags_; }

inline unsigned char
d_Log::d_syslog() { return syslog_; }

inline unsigned long
d_Log::d_size() { return size_; }

inline unsigned char
d_Log::d_what(const unsigned char what)
{ unsigned char w = what_; what_ = what; return w; }

inline unsigned char
d_Log::d_flags(const unsigned char flags)
{ unsigned char f = flags_; flags_ = flags; return f; }

inline unsigned char
d_Log::d_syslog(const unsigned char syslog)
{ unsigned char s = syslog_; syslog_ = syslog; return s; }

inline unsigned long
d_Log::d_size(const unsigned long size)
{ unsigned long s = size_; size_ = size; return s; }

#endif /* __d_Log_h__ */

