/*

Simplified BSD License
taken from
<http://www.opensource.org/licenses/bsd-license.php> 

Copyright (c) 2008, Saleem N. Bhatti <http://www.cs.st-andrews.ac.uk/~saleem/>
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  - Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  - Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

  - Neither the name of 'Saleem N. Bhatti' or 'Saleem Bhatti', nor the
    names of any other contributors may be used to endorse or promote
    products derived from this software without specific prior written
    permission.

DISCLAIMER
----------
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

27 November 2008

*/

/******************************************************************************
Some logging stuff

saleem <http://www.cs.st-andrews.ac.uk/~saleem/>
November 2008
November 2004
November 2001
March 1993
******************************************************************************/

#ifndef __d_log_c_h__
#define __d_log_c_h__

#include <stdio.h>

#if defined(__cplusplus)
extern "C" {
#endif

typedef struct d_Log_s {

  char *program;       /* program name */
  char *name;          /* name of file */
  FILE *log;           /* non-NULL after call to d_log_open() */

                         /* use the following two for what, flags and syslog */
#define D_LOG_all        ((unsigned char) 0xff)
#define D_LOG_none       ((unsigned char) 0x00)

  unsigned char what;  /* what to log - use as you like */
#define D_LOG_debug      ((unsigned char) 0x01)
#define D_LOG_advise     ((unsigned char) 0x02)
#define D_LOG_error      ((unsigned char) 0x04)
#define D_LOG_trace      ((unsigned char) 0x08)
#define D_LOG_exceptions ((unsigned char) 0x10)
#define D_LOG_fatal      ((unsigned char) 0x20)

  unsigned char syslog; /* takes same values as what */

  unsigned char flags;
#define D_LOG_log         ((unsigned char) 0x01) /* log to file 'name' */
#define D_LOG_stderr      ((unsigned char) 0x02) /* log to stderr */
#define D_LOG_close       ((unsigned char) 0x04) /* close between writes */
#define D_LOG_alert       ((unsigned char) 0x08) /* d_log() errors to stderr */
#define D_LOG_time        ((unsigned char) 0x10) /* time */
#define D_LOG_date        ((unsigned char) 0x20) /* date */
#define D_LOG_user        ((unsigned char) 0x40) /* user id */

  unsigned long size;   /* max log size (bytes) - 0 means no size limit */

} d_Log_t;

/* Initialisation macro */
#define INITIALISE_D_LOG(d_, p_, n_, w_, s_, f_, sz_) \
    (d_)->program = (p_), (d_)->name = (n_), (d_)->log = (FILE *) 0, \
    (d_)->what = (w_), (d_)->syslog = (s_), \
    (d_)->flags = (f_), (d_)->size = sz_

/* Handy initialise macro */
#define D_LOG(d_, p_, n_) \
        INITIALISE_D_LOG(d_, p_, n_, D_LOG_all, D_LOG_none, D_LOG_all, 0)

/* Do not use d_log_open() and d_log_close() if using the D_LOG_close flag */
int d_log_open(d_Log_t *log);
int d_log_close(d_Log_t *log);
int d_log(d_Log_t *log, const unsigned char what, const char *format, ...);

#if defined(__cplusplus)
}
#endif

#endif /* __d_log_h__ */

